@@ -1,9 +1,8 @@
 /******************************************************************************
- * $Id: geo_extra.c,v 1.2 1999/05/04 03:09:33 warmerda Exp $
+ * $Id$
  *
  * Project:  libgeotiff
- * Purpose:  Code to normalize a few common PCS values without use of CSV
- *           files.
+ * Purpose:  Code to normalize a few common PCS values
  * Author:   Frank Warmerdam, warmerda@home.com
  *
  ******************************************************************************
@@ -26,16 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: geo_extra.c,v $
- * Revision 1.2  1999/05/04 03:09:33  warmerda
- * avoid warnings
- *
- * Revision 1.1  1999/04/28 20:01:29  warmerda
- * new
- *
- */
+ *****************************************************************************/
 
 /*
 #include "geotiff.h"
@@ -46,7 +36,7 @@
 #include "geo_normalize.h"
 #include "geovalues.h"
 
-static int StatePlaneTable[] = 
+static const int StatePlaneTable[] =
 {
     PCS_NAD83_Alabama_East,		Proj_Alabama_CS83_East,
     PCS_NAD83_Alabama_West,		Proj_Alabama_CS83_West,
@@ -218,7 +208,7 @@ static int StatePlaneTable[] =
     PCS_NAD83_Wyoming_E_Cen,		Proj_Wyoming_CS83_East_Central,
     PCS_NAD83_Wyoming_W_Cen,		Proj_Wyoming_CS83_West_Central,
     PCS_NAD83_Wyoming_West,		Proj_Wyoming_CS83_West,
-    
+
     PCS_NAD83_Puerto_Rico_Virgin_Is,	Proj_Puerto_Rico_Virgin_Is,
 
     PCS_NAD27_Alabama_East,		Proj_Alabama_CS27_East,
@@ -396,7 +386,7 @@ static int StatePlaneTable[] =
     PCS_NAD27_Wyoming_E_Cen,		Proj_Wyoming_CS27_East_Central,
     PCS_NAD27_Wyoming_W_Cen,		Proj_Wyoming_CS27_West_Central,
     PCS_NAD27_Wyoming_West,		Proj_Wyoming_CS27_West,
-    
+
     PCS_NAD27_Puerto_Rico,		Proj_Puerto_Rico_CS27,
 
     KvUserDefined
@@ -441,12 +431,15 @@ int	GTIFMapSysToPCS( int MapSys, int Datum, int nZone )
 	int		i;
 
         PCSCode = 10000 + nZone;
-
 	for( i = 0; StatePlaneTable[i] != KvUserDefined; i += 2 )
 	{
 	    if( StatePlaneTable[i+1] == PCSCode )
 	        PCSCode = StatePlaneTable[i];
 	}
+
+        /* Old EPSG code was in error for Tennesse CS27, override */
+        if( nZone == 4100 )
+            PCSCode = 2204;
     }
     else if( MapSys == MapSys_State_Plane_83 )
     {
@@ -459,9 +452,13 @@ int	GTIFMapSysToPCS( int MapSys, int Datum, int nZone )
 	    if( StatePlaneTable[i+1] == PCSCode )
 	        PCSCode = StatePlaneTable[i];
 	}
+
+        /* Old EPSG code was in error for Kentucky North CS83, override */
+        if( nZone == 1601 )
+            PCSCode = 2205;
     }
 
-    return( PCSCode );
+    return PCSCode;
 }
 
 /************************************************************************/
@@ -487,13 +484,21 @@ int	GTIFMapSysToProj( int MapSys, int nZone )
     else if( MapSys == MapSys_State_Plane_27 )
     {
         ProjCode = 10000 + nZone;
+
+        /* Tennesse override */
+        if( nZone == 4100 )
+            ProjCode = 15302;
     }
     else if( MapSys == MapSys_State_Plane_83 )
     {
         ProjCode = 10000 + nZone + 30;
+
+        /* Kentucky North override */
+        if( nZone == 1601 )
+            ProjCode = 15303;
     }
 
-    return( ProjCode );
+    return ProjCode;
 }
 
 /************************************************************************/
@@ -526,9 +531,9 @@ int	GTIFMapSysToProj( int MapSys, int nZone )
  * The datum (really this is the GCS) is set to a GCS_ value such as GCS_NAD27.
  *
  * This function is useful to recognise (most) UTM and State Plane coordinate
- * systems, even if CSV files aren't available to translate them automatically.
+ * systems.
  * It is used as a fallback mechanism by GTIFGetDefn() for normalization when
- * CSV files aren't found. 
+ * PROJ database is not found.
  */
 
 int GTIFPCSToMapSys( int PCSCode, int * pDatum, int * pZone )
@@ -547,7 +552,7 @@ int GTIFPCSToMapSys( int PCSCode, int * pDatum, int * pZone )
 	Proj = MapSys_UTM_North;
 	nZone = PCSCode - PCS_NAD27_UTM_zone_3N + 3;
     }
-    else if( PCSCode >= PCS_NAD83_UTM_zone_3N 
+    else if( PCSCode >= PCS_NAD83_UTM_zone_3N
 	     && PCSCode <= PCS_NAD83_UTM_zone_23N )
     {
 	Datum = GCS_NAD83;
@@ -599,7 +604,7 @@ int GTIFPCSToMapSys( int PCSCode, int * pDatum, int * pZone )
 	Proj = MapSys_UTM_South;
 	nZone = PCSCode - PCS_WGS84_UTM_zone_1S + 1;
     }
-    else if( PCSCode >= PCS_SAD69_UTM_zone_18N 
+    else if( PCSCode >= PCS_SAD69_UTM_zone_18N
 	     && PCSCode <= PCS_SAD69_UTM_zone_22N )
     {
 	Datum = KvUserDefined;
@@ -636,7 +641,7 @@ int GTIFPCSToMapSys( int PCSCode, int * pDatum, int * pZone )
             Proj = MapSys_State_Plane_27;
 	    Datum = GCS_NAD27;
         }
-	
+
 	nZone = PCSCode - 10000;
 	if( Datum == GCS_NAD83 )
 	    nZone -= 30;
@@ -648,7 +653,7 @@ int GTIFPCSToMapSys( int PCSCode, int * pDatum, int * pZone )
     if( pZone != NULL )
         *pZone = nZone;
 
-    return( Proj );
+    return Proj;
 }
 
 /************************************************************************/
@@ -701,7 +706,7 @@ int GTIFProjToMapSys( int ProjCode, int * pZone )
     }
 
 /* -------------------------------------------------------------------- */
-/*      Handle State Plane.  I think there are some anomolies in        */
+/*      Handle State Plane.  I think there are some anomalies in        */
 /*      here, so this is a bit risky.                                   */
 /* -------------------------------------------------------------------- */
     else if( ProjCode >= 10101 && ProjCode <= 15299 )
@@ -717,10 +722,9 @@ int GTIFProjToMapSys( int ProjCode, int * pZone )
             nZone = ProjCode - 10000;
         }
     }
-    
+
     if( pZone != NULL )
         *pZone = nZone;
 
-    return( MapSys );
+    return MapSys;
 }
-
